### Source codes for normalization to remove technical noise for single cell RNA sequence data and inference on biological gene expression variability
### 
### Based on:
###
### Lina Zheng, Bo Ding and Wei Wang
###
### Single cell has been a powerful and popular technology to explore heterogeneous gene expressions  among single cells. It may reveal the magic biological
### mechanism for individual gene function.  

## Load required R packages
# It needs MASS, gplots, ggplot2 packages
library(MASS)
library(gplots)
library(ggplot2)
library(ctc)
source("include/GRM_lib.r")

args <-commandArgs(trailingOnly=TRUE)
input1=paste("ERCC information:",args[1],sep=" ")
print (input1)
input2=paste("Data:",args[2],sep=" ")
print (input2)
input3=paste("Outpath:",args[3],sep=" ")
print (input3)
outpath=args[3]
## define data objects:
# required data objects:
# dataset1(express), ERCC_standard

# 1. vector with spike-in numbers. Names should correspond to spike-in names
# ercc file: the add-in standard ERCC concentration, gotten from the experiments directly
# file format: each row is each ERCC, each column is each sample
#           the first column is ERCC name, the second column is ERCC standard concentration.
#           especially sorted by the ERCC names
ercc <- read.csv(args[1],header=TRUE)
spike=ercc[,1]
spike=as.character(spike)

# 2. Load expression data (Each column should be one sample, Each row should be one gene
#    across all the samples. Especially, the first row is file name, the second row is cell 
#    cell types, for example different stages of cells)

# read data
#dataset1=read.csv("data/filename_and_type.csv", header=F)
dataset2=read.csv(args[2], header=T)
# dataset clean for gamma regression input
genename=dataset2[,1]
genename=as.character(genename)
dataset=dataset2[,-1]
rownames(dataset)=genename

# index file name
filename=colnames(dataset2)
filename=filename[-1]
filename=as.matrix(filename)
filename=as.character(filename)
# optional, if there are any celltypes for samples available
## 3. Split ERCC and gene data

# ERCC
# ercc_response file: the fpkm reads for ERCC across all the samples
#             format: each row is each ERCC, each column is each sample
#                     especially sorted by the ERCC names
response_data=dataset[rownames(dataset) %in% spike,]
ercc_response=response_data[order(rownames(response_data)),]

# Gene
# gene file: the fpkm reads for genes across all the sampels
#    format: each row is each gene, each column is each sample
#            especially the column names should be in the same order with ercc_response file
gene=dataset[! rownames(dataset) %in% spike,]

## 4. Gamma regression
regresult=gammareg(ercc_response, ercc, gene, filename, SE=F)

# output result
#errorset=regresult$errorset
predictset=regresult$predictset
#errorfile=paste(outpath,"errorset.csv",sep="/")
predictfile=paste(outpath,"predict.csv",sep="/")
#write.csv(errorset, errorfile)
write.csv(predictset,predictfile)

## 5. Result analysis
# select genes and samples:
# 124 Sftpc+ cells from distal mouse lung epithelium of embryonic
# (E14.5,   e16.5, e18.5) and adult mice based on genes detected
# in more than two cells.
# Details for how to select genes and samples, please refer to "Reconstructing lineage hierarchies
# of the distal lung epithelium using single-cell RNA-seq", Nature,509, 371-375
# Supplementary Data 2 (22 KB)

#select_norm=read.csv("data/all_fit_lung_sort.csv")
#select_unnorm=read.csv("data/all_select_lung_sort.csv")

# PCA
#pca_unnorm_result=pca_plot(select_unnorm, plot=T)
#pca_norm_result=pca_plot(select_norm, plot=T)
# Hierachical cluster

#cluster_result_norm=cluster(select_norm, treeview=T)
#result11=cluster_result_norm$hcresult
#plot(result11)
#result12=cluster_result_norm$treeview
#write.table(result12,"output/genes with predict result hclust output.txt")

#cluster_result_unnorm=cluster(select_unnorm, treeview=T)
#result21=cluster_result_unnorm$hcresult
#plot(result21)
#result22=cluster_result_unnorm$treeview
#write.table(result22,"output/genes with FPKM hclust output.txt")

